#include<routines.hpp>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<fields.hpp>
#include<math.h>

void read_input_file(struct ParticleParameters* InitialParameters, struct SimulationParameters* SimPar, char* input_file_name, char* output_file_name)
{
  FILE* input_fp = fopen(input_file_name, "r");
  char temp[1000];
  char first_character;
  char* token;
  char x0_str[] = "x0";
  char y0_str[] = "y0";
  char z0_str[] = "z0";
  char q_str[] = "q";
  char m_str[] = "m";
  char vx0_str[] = "vx0";
  char vy0_str[] = "vy0";
  char vz0_str[] = "vz0";
  char dt_str[] = "dt";
  char tmax_str[] = "tmax";
  char output_file_str[] = "output_file";
  char N_print_str[] = "N_print";
    
  if (input_fp == NULL)
    {
      printf("Cannot find %s. Program aborted\n", input_file_name);
      exit(1);
    }
  
  printf("Reading input file name: %s\n", input_file_name);
  
  while (!feof(input_fp))
    {
      fgets(temp, sizeof(temp), input_fp);
      //printf("Reading line: %s\n", temp);
      first_character = temp[0];
      if (first_character != '#')   // Lines starting with # are comments
	{
	  token = strtok(temp, " ");
	  if (strcmp(token, q_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      InitialParameters->charge = atof(token);
	    }
	  if (strcmp(token, m_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      InitialParameters->mass = atof(token);
	    }
	  if (strcmp(token, x0_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      InitialParameters->X0.SetVx(atof(token));
	    }
	  if (strcmp(token, y0_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      InitialParameters->X0.SetVy(atof(token));
	    }
	  if (strcmp(token, z0_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      InitialParameters->X0.SetVz(atof(token));
	    }
	  if (strcmp(token, vx0_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      InitialParameters->V0.SetVx(atof(token));
	    }
	  if (strcmp(token, vy0_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      InitialParameters->V0.SetVy(atof(token));
	    }
	  if (strcmp(token, vz0_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      InitialParameters->V0.SetVz(atof(token));
	    }
	  if (strcmp(token, dt_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      SimPar->dt = atof(token);
	    }
	  if (strcmp(token, tmax_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      SimPar->tmax = atof(token);
	    }
	  if (strcmp(token, output_file_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      strcpy(output_file_name, token);
	    }
	  if (strcmp(token, N_print_str) == 0)
	    {
	      token = strtok(NULL, " ");
	      SimPar->N_print = atoi(token);
	    }
	}
    }
  
  printf("Following orbit with the following parameters:\n");
  printf("Charge (C) = %e\n", InitialParameters->charge);
  printf("Mass (kg) = %e\n", InitialParameters->mass);
  printf("x0 (m) = %e\n", InitialParameters->X0.GetVx());
  printf("y0 (m) = %e\n", InitialParameters->X0.GetVy());
  printf("z0 (m) = %e\n", InitialParameters->X0.GetVz());
  printf("vx0 (m/s) = %e\n", InitialParameters->V0.GetVx());
  printf("vy0 (m/s) = %e\n", InitialParameters->V0.GetVy());
  printf("vz0 (m/s) = %e\n", InitialParameters->V0.GetVz());
  printf("Integration step (s) = %e\n", SimPar->dt);
  printf("Max following time (s) = %e\n", SimPar->tmax);
  printf("Writing trajectory to file: %s\n", output_file_name);
  printf("Print trajectory every %d points\n", SimPar->N_print);
  fclose(input_fp);
  return;
}
	      
void integrate_trajectory(struct ParticleParameters* InitialParameters, struct SimulationParameters* SimPar, char* output_file_name)
{
  // This is the method of Birdsall and Langdon 1985; see Bellan, ex. 7, ch. 1
  
  printf("Integrating trajectory...\n");
  double q = InitialParameters->charge;
  double m = InitialParameters->mass;
  double dt = SimPar->dt;
  Vector V0(InitialParameters->V0.GetVx(), InitialParameters->V0.GetVy(), InitialParameters->V0.GetVz());
  Vector X0(InitialParameters->X0.GetVx(), InitialParameters->X0.GetVy(), InitialParameters->X0.GetVz());
  double tmax = SimPar->tmax;
  int N_print = SimPar->N_print;
  int i_iter = 0;  // iteration number
  Vector Vold;
  Vector Vnew;
  Vector Xold;
  Vector Xnew;
  Vector B;
  Vector Omega;
  Vector C;
  Vector A;
  Vector E;
  Vector Sigma;
  double AdotC;
  Vector AvecC;
  double A2;
  double Energy = 0.5 * m * V0.Modulus() * V0.Modulus();
  double t = 0.0;
  Vold = V0;
  Xold = X0;
  B = MagneticField(X0, t);
  double pitch = (V0 * B);
  pitch = pitch / V0.Modulus();
  pitch = pitch / B.Modulus();
  double sinPitch = sqrt(1.0 - pitch * pitch);
  double mu = m * V0.Modulus()*V0.Modulus() * sinPitch * sinPitch / (2.0 * B.Modulus()); 
  
  FILE* trajectory_fp;
  trajectory_fp = fopen(output_file_name, "w");
  fprintf(trajectory_fp, "# Time (s)   x (m)   y (m)   z (m)   vx (m/s)   vy (m/s)   vz (m/s)   Energy (J)   cos(pitch angle)  mu (A*m^2) mu*B (J)\n");
  fprintf(trajectory_fp, "%e %e %e %e %e %e %e %e %e %e %e\n", t, X0.GetVx(), X0.GetVy(), X0.GetVz(), V0.GetVx(), V0.GetVy(), V0.GetVz(), Energy, pitch, mu, mu*B.Modulus());
  
  while (t < tmax)
    {
      i_iter += 1;
      B = MagneticField(Xold, t);
      E = ElectricField(Xold, t);
      Sigma = q * E;
      Sigma = Sigma / m;
      Omega = q * B;
      Omega = Omega / m;
      A = Omega * dt;
      A = A / 2.0;
      C = VectorProduct(Vold, Omega);
      C = C / 2.0;
      C = C + Sigma;
      C = C * dt;
      C = Vold + C;
      AdotC = A * C;
      AvecC = VectorProduct(A,C);
      A2 = A * A;
      Vnew = AdotC * A;
      Vnew = Vnew - AvecC;
      Vnew = Vnew + C;
      Vnew = Vnew / (1.0 + A2);
      Xnew = Vnew * dt;
      Xnew = Xnew + Xold;
      Energy = 0.5 * m * Vnew.Modulus() * Vnew.Modulus();
      pitch = Vnew * B;
      pitch = pitch / B.Modulus();
      pitch = pitch / Vnew.Modulus();
      sinPitch = sqrt(1.0 - pitch*pitch);
      mu = m * Vnew.Modulus()*Vnew.Modulus() * sinPitch * sinPitch / (2.0 * B.Modulus()); 
      Xold = Xnew;
      Vold = Vnew;
      t = t + dt;
      if (i_iter % N_print == 0)
	{
	  fprintf(trajectory_fp, "%e %e %e %e %e %e %e %e %e %e %e\n", t, Xnew.GetVx(), Xnew.GetVy(), Xnew.GetVz(), Vnew.GetVx(), Vnew.GetVy(), Vnew.GetVz(), Energy, pitch, mu, mu*B.Modulus());
	}
    }
  printf("Trajectory integration ended\n");
  printf("Output in: %s\n", output_file_name);
  fclose(trajectory_fp);
  return;
}

